
const fs = require('fs')
const path = require('path')
const iconv = require('iconv-lite')

module.exports.type = 'startup'
module.exports.meta = {
  name: 'dat_mirror_test',
  description: '特定のディレクトリ以下に掲示板のdat構造を作成',
  version: '1.0.0',
  needVersion: '0.24.8'
}

// dat構造を作成する場所
// 指定しない場合は ${log_dir}/Logs以下に展開されます
const LOGS_DIR = ''

const WDay = ['日', '月', '火', '水', '木', '金', '土']

let timeout = {}

module.exports.mainScript = async (settings, tools) => {
  const { config, userSettings } = settings
  const { emitter } = tools
  const base_dir = LOGS_DIR || path.join(config['log_dir'], 'Logs')
  const log_dir = config['log_dir']

  const generate_2chdat = async (uo) => {
    const datpath = path.join(log_dir, uo.sitename, uo.board, uo.threadkey + '.json')
    const buf = await fs.promises.readFile(datpath, { encoding: 'utf8' })
    const js = JSON.parse(buf.toString())
    let dat_array = []
    let thread_title = js.title
    for (let l of js.thread_array) {
      let name = l.mname
      if (l.trip) {
        name = `${name} </b>◆${l.trip} <b>`
      }
      if (l.nickname) {
        name = `${name} </b>(${l.nickname}${l.id2 ? ` ${l.id2}` : ''})<b>`
      }
      let mail = l.mail
      if (!l.timestamp) {
        dat_array.push(['Aborn', 'Aborn', 'Aborn', 'Aborn'].join('<>'))
      } else {
        let date = new Date(l.timestamp)
        let dateStr = new Intl.DateTimeFormat('jp', {
          year: 'numeric', month: '2-digit', day: '2-digit',
          hour: '2-digit',
          minute: '2-digit',
          second: '2-digit',
          fractionalSecondDigits: 2,
        }).format(date)
        dateStr = dateStr.replace(/ /, `(${WDay[date.getDay()]}) `)

        if (l.id_verbose) {
          dateStr = `${dateStr} ID:${l.id_verbose}`
        }

        let text = l.body
        text = text.replace(/\s*<br>\s*/g, '\n')
        text = ' ' + text.replace(/<("[^"]*"|'[^']*'|[^'">])*>/g, '') + ' '
        text = text.replace(/>>/g, '&gt;&gt;')
        text = text.replace(/\n/g, ' <br> ')
        dat_array.push([
          name, mail, dateStr, text, ''
        ].join('<>'))
      }
    }
    dat_array[0] = dat_array[0] + thread_title
    return dat_array.join('\n')
  }

  emitter.on('ThreadFetchSuccess', async (uo) => {
    clearTimeout(timeout[uo.location])
    timeout[uo.location] = setTimeout(async () => {
      try {
        const dir = path.join(base_dir, uo.sitename, uo.board)
        const datname = uo.threadkey
        const dat = await generate_2chdat(uo)
        await fs.promises.mkdir(dir, { recursive: true })
        await fs.promises.writeFile(path.join(dir, `${datname}.dat`), iconv.encode(dat, 'cp932'))
      } catch (e) {
        console.log(e)
      }
    }, 500)
  })
}
